#!/bin/sh

###############################################################################
##
##  ESM EXAMPLE
##
###############################################################################
# cd to the directory where this script is
cd `echo $0 | sed 's/\(.*\)\/.*/\1/'` # extract pathname
EXAMPLE_DIR=`pwd`

# check whether echo has the -e option
if test "`echo -e`" = "-e" ;
then ECHO=echo ;
else ECHO="echo -e" ;
fi

$ECHO "$EXAMPLE_DIR : starting"
$ECHO "This example shows how to use ESM to calculate Al(111) and H2O"
$ECHO "using the three available sets of boundary conditions."

# set the needed environment variables
export PW_ROOT=/opt/QE61
export BIN_DIR=${PW_ROOT}/bin
export PSEUDO_DIR=/home/jyhuang/WS_QE/pseudo
export TMP_DIR=/home/jyhuang/WS_QE/scratch
export PARA_PREFIX='mpirun -np 4 --map-by core'
export PARA_POSTFIX=''

# required executables and pseudopotentials
BIN_LIST="pw.x"
PSEUDO_LIST="Al.pbe-rrkj.UPF H.pbe-rrkjus.UPF O.pbe-rrkjus.UPF"

$ECHO "  executables directory: $BIN_DIR"
$ECHO "  pseudo directory:      $PSEUDO_DIR"
$ECHO "  temporary directory:   $TMP_DIR"
$ECHO "  checking that needed directories and files exist...\c"

# check for directories
for DIR in "$BIN_DIR" "$PSEUDO_DIR" ; do
    if test ! -d $DIR ; then
        $ECHO
        $ECHO "ERROR: $DIR not existent or not a directory"
        $ECHO "Aborting"
        exit 1
    fi
done
for DIR in "$TMP_DIR" "$EXAMPLE_DIR/results" ; do
    if test ! -d $DIR ; then
        mkdir $DIR
    fi
done
cd $EXAMPLE_DIR/results

# check for executables
for FILE in $BIN_LIST ; do
    if test ! -x $BIN_DIR/$FILE ; then
        $ECHO
        $ECHO "ERROR: $BIN_DIR/$FILE not existent or not executable"
        $ECHO "Aborting"
        exit 1
    fi
done

# check for pseudopotentials
for FILE in $PSEUDO_LIST ; do
    if test ! -r $PSEUDO_DIR/$FILE ; then
       $ECHO
       $ECHO "Downloading $FILE to $PSEUDO_DIR...\c"
            $WGET $PSEUDO_DIR/$FILE $NETWORK_PSEUDO/$FILE 2> /dev/null
    fi
    if test $? != 0; then
        $ECHO
        $ECHO "ERROR: $PSEUDO_DIR/$FILE not existent or not readable"
        $ECHO "Aborting"
        exit 1
    fi
done
$ECHO " done"

# how to run executables
# Run pw.x on muticore cpu
PW_COMMAND="$PARA_PREFIX $BIN_DIR/pw.x $PARA_POSTFIX"
$ECHO "  running pw.x     as: $PW_COMMAND"

# run by pw-gpu.x on multicore cpu and gpu
# export PHI_DGEMM_SPLIT=0.98
# export PHI_ZGEMM_SPLIT=0.98
# PW_COMMAND="$PARA_PREFIX $BIN_DIR/pw-gpu.x $PARA_POSTFIX"
# $ECHO "  running pw-gpu.x     as: $PW_COMMAND"

start_time=`date +%s`
echo $start_time
# scf calculation for H2O with no ESM
# ibrav = 6: Tetragonal P (st), celldm(3)=c/a
# assume_isolated= esm: isolated (a molecule or a cluster in a 3D supercell).
# For polarized or charged slab calculation, embeds the simulation cell within
# an effective semi-infinite medium in the perpendicular direction (along z).
# esm_bc='pbc': If assume_isolated = 'esm', use the pbc for either side of the slab.
cat > H2O.noesm.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='H2O',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 6,
    celldm(1) =20.0,
    celldm(3) = 1.200,
    nat= 3,
    ntyp= 2,
    ecutwfc = 25.0,
    ecutrho = 200.0,
    occupations='smearing',
    smearing='gaussian',
    degauss=0.05,
    assume_isolated = 'esm',
    esm_bc='pbc'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 H  1.00794  H.pbe-rrkjus.UPF
 O  55.847   O.pbe-rrkjus.UPF
ATOMIC_POSITIONS
O        0.000000000   0.500000000   0.000000000    0   0   0
H        0.043138813   0.431028632   0.043078297    0   0   0
H        0.036635422   0.576406407   0.035949164
K_POINTS gamma
EOF

$ECHO "  running the scf calculation for H2O without ESM...\c"
$PW_COMMAND < H2O.noesm.in > H2O.noesm.out
# bash check_failure $?
$ECHO " done"

# scf calculation for H2O with ESM bc1 (vacuum-slab-vacuum)
# esm_bc: If assume_isolated = 'esm', use the bc1 for either side of the slab.
cat > H2O.bc1.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='H2O',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 6,
    celldm(1) =20.0,
    celldm(3) = 1.200,
    nat= 3,
    ntyp= 2,
    ecutwfc = 25.0,
    ecutrho = 200.0,
    occupations='smearing',
    smearing='gaussian',
    degauss=0.05,
    assume_isolated = 'esm',
    esm_bc='bc1'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 H  1.00794  H.pbe-rrkjus.UPF
 O  55.847   O.pbe-rrkjus.UPF
ATOMIC_POSITIONS
O        0.000000000   0.500000000   0.000000000    0   0   0
H        0.043138813   0.431028632   0.043078297    0   0   0
H        0.036635422   0.576406407   0.035949164
K_POINTS gamma
EOF

$ECHO "  running the scf calculation for H2O with ESM bc1 (vacuum-slab-vacuum)...\c"
$PW_COMMAND < H2O.bc1.in > H2O.bc1.out
# bash check_failure $?
$ECHO " done"

# clean TMP_DIR
$ECHO "  cleaning $TMP_DIR...\c"
rm -rf $TMP_DIR/H2O*
$ECHO " done"

# scf calculation for Al(111) with ESM bc2 (metal-slab-metal), no field
# ibrav = 0: free system with axis provided by celldime(1).
cat > Al111.bc2.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='Al111',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 0,
    celldm(1) = 7.653393855,
    nat= 7,
    ntyp= 1,
    ecutwfc = 20.0,
    nosym=.TRUE.
    occupations='smearing',
    smearing='mp',
    degauss=0.05
    assume_isolated='esm',
    esm_bc='bc2'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 Al  26.981538   Al.pbe-rrkj.UPF
CELL_PARAMETERS hexagonal
 0.707106781    0.000000000  0.000000000
 0.353553390    0.612372436  0.000000000
 0.000000000    0.000000000 10.000000000
ATOMIC_POSITIONS angstrom
Al 0.00000    0.00000   -7.01481
Al 0.00000    1.65341   -4.67654
Al 1.43189    0.82670   -2.33827
Al 0.00000    0.00000    0.00000
Al 0.00000    1.65341    2.33827
Al 1.43189    0.82670    4.67654
Al 0.00000    0.00000    7.01481
 K_POINTS automatic
 8 8 1 0 0 0
EOF

$ECHO "  running the scf calculation for Al(111) with ESM bc2 (metal-slab-metal)"
$ECHO "  (no applied field)...\c"
$PW_COMMAND < Al111.bc2.in > Al111.bc2.out
# bash check_failure $?
$ECHO " done"

# scf calculation for Al(111) with ESM bc2 (metal-slab-metal), with field
cat > Al111.bc2_efield.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='Al111',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 0,
    celldm(1) = 7.653393855,
    nat= 7,
    ntyp= 1,
    ecutwfc = 20.0,
    nosym=.TRUE.
    occupations='smearing',
    smearing='mp',
    degauss=0.05
    assume_isolated='esm',
    esm_bc='bc2',
    esm_efield=0.00192148511256006
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 Al  26.981538   Al.pbe-rrkj.UPF
CELL_PARAMETERS hexagonal
 0.707106781    0.000000000  0.000000000
 0.353553390    0.612372436  0.000000000
 0.000000000    0.000000000 10.000000000
ATOMIC_POSITIONS angstrom
Al 0.00000    0.00000   -7.01481
Al 0.00000    1.65341   -4.67654
Al 1.43189    0.82670   -2.33827
Al 0.00000    0.00000    0.00000
Al 0.00000    1.65341    2.33827
Al 1.43189    0.82670    4.67654
Al 0.00000    0.00000    7.01481
 K_POINTS automatic
 8 8 1 0 0 0
EOF

$ECHO "  running the scf calculation for Al(111) with ESM bc2 (metal-slab-metal)"
$ECHO "  with applied electric field...\c"
$PW_COMMAND < Al111.bc2_efield.in > Al111.bc2_efield.out
# bash check_failure $?
$ECHO " done"

# scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal), uncharged
cat > Al111.bc3.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='Al111',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 0,
    celldm(1) = 7.653393855,
    nat= 7,
    ntyp= 1,
    ecutwfc = 20.0,
    nosym=.TRUE.
    occupations='smearing',
    smearing='mp',
    degauss=0.05
    assume_isolated='esm',
    esm_bc='bc3'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 Al  26.981538   Al.pbe-rrkj.UPF
CELL_PARAMETERS hexagonal
 0.707106781    0.000000000  0.000000000
 0.353553390    0.612372436  0.000000000
 0.000000000    0.000000000 10.000000000
ATOMIC_POSITIONS angstrom
Al 0.00000    0.00000   -7.01481
Al 0.00000    1.65341   -4.67654
Al 1.43189    0.82670   -2.33827
Al 0.00000    0.00000    0.00000
Al 0.00000    1.65341    2.33827
Al 1.43189    0.82670    4.67654
Al 0.00000    0.00000    7.01481
 K_POINTS automatic
 8 8 1 0 0 0
EOF

$ECHO "  running the scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal)"
$ECHO "  (neutral, uncharged)...\c"
$PW_COMMAND < Al111.bc3.in > Al111.bc3.out
# bash check_failure $?
$ECHO " done"

# scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal), - charged
# specified by tot_charge = -0.005 in &system
cat > Al111.bc3_m005.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='Al111',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 0,
    celldm(1) = 7.653393855,
    tot_charge = -0.005,
    nat= 7,
    ntyp= 1,
    ecutwfc = 20.0,
    nosym=.TRUE.
    occupations='smearing',
    smearing='mp',
    degauss=0.05
    assume_isolated='esm',
    esm_bc='bc3'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 Al  26.981538   Al.pbe-rrkj.UPF
CELL_PARAMETERS hexagonal
 0.707106781    0.000000000  0.000000000
 0.353553390    0.612372436  0.000000000
 0.000000000    0.000000000 10.000000000
ATOMIC_POSITIONS angstrom
Al 0.00000    0.00000   -7.01481
Al 0.00000    1.65341   -4.67654
Al 1.43189    0.82670   -2.33827
Al 0.00000    0.00000    0.00000
Al 0.00000    1.65341    2.33827
Al 1.43189    0.82670    4.67654
Al 0.00000    0.00000    7.01481
 K_POINTS automatic
 8 8 1 0 0 0
EOF

$ECHO "  running the scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal)"
$ECHO "  (-0.005e charged)...\c"
$PW_COMMAND < Al111.bc3_m005.in > Al111.bc3_m005.out
# bash check_failure $?
$ECHO " done"

# scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal), + charged
cat > Al111.bc3_p005.in << EOF
 &control
    calculation='scf',
    restart_mode='from_scratch',
    prefix='Al111',
    pseudo_dir = '$PSEUDO_DIR/',
    outdir='$TMP_DIR/',
    tprnfor = .TRUE.
 /
 &system
    ibrav = 0,
    celldm(1) = 7.653393855,
    tot_charge = 0.005,
    nat= 7,
    ntyp= 1,
    ecutwfc = 20.0,
    nosym=.TRUE.
    occupations='smearing',
    smearing='mp',
    degauss=0.05
    assume_isolated='esm',
    esm_bc='bc3'
 /
 &electrons
    mixing_beta = 0.5
 /
ATOMIC_SPECIES
 Al  26.981538   Al.pbe-rrkj.UPF
CELL_PARAMETERS hexagonal
 0.707106781    0.000000000  0.000000000
 0.353553390    0.612372436  0.000000000
 0.000000000    0.000000000 10.000000000
ATOMIC_POSITIONS angstrom
Al 0.00000    0.00000   -7.01481
Al 0.00000    1.65341   -4.67654
Al 1.43189    0.82670   -2.33827
Al 0.00000    0.00000    0.00000
Al 0.00000    1.65341    2.33827
Al 1.43189    0.82670    4.67654
Al 0.00000    0.00000    7.01481
 K_POINTS automatic
 8 8 1 0 0 0
EOF

$ECHO "  running the scf calculation for Al(111) with ESM bc3 (vacuum-slab-metal)"
$ECHO "  (+0.005e charged)...\c"
$PW_COMMAND < Al111.bc3_p005.in > Al111.bc3_p005.out
# bash check_failure $?
$ECHO " done"

end_time=`date +%s`
elaps=$(($end_time - $start_time))
echo "Entire ESM simulation took $elaps seconds to complete"

# clean TMP_DIR
$ECHO "  cleaning $TMP_DIR...\c"
rm -rf $TMP_DIR/Al111*
$ECHO " done"
$ECHO "$EXAMPLE_DIR: done"
